#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""
run 命令的参数解析
"""
import os
from wpy.argument import Action
from wpy.argument import CommandArgumentParserFactory

from wush.common import utils
from wush.common.constants import Constants
from wush.common.utils import run_shell
from wush.common.loggers import create_logger
from wush.common.run_mode import RUN_MODE
from wush.config import load_config
from wush.web.request import RequestClient
from wush.web.request import RequestBuilder
from wush.web.history import History

from .command import CmdArgumentParser


@CommandArgumentParserFactory.register()
class RunArgumentParser(CmdArgumentParser):
    cmd = 'run'
    logger = create_logger('RunArgumentParser')

    @classmethod
    def default(cls):
        """
        初始化一个实例
        """
        item = cls()
        item.add_argument('cmd')
        item.add_argument('-m', '--module', help='模块名称')
        item.add_argument('-s', '--space', help='空间名称')
        item.add_argument('-n', '--name', help='请求名称')
        item.add_argument('--params', action = Action.APPEND.value,
            help='请求地址参数')
        item.add_argument('--json', action = Action.APPEND.value,
            help='请求 body 参数，json 格式')
        item.add_argument('--open', action = Action.STORE_TRUE.value,
            help = '是否通过浏览器打开请求结果')
        item.add_argument('--curl', action = Action.STORE_TRUE.value,
            help = '是否使用 curl 文本')
        item.add_argument('--with-browser-cookie',
            action = Action.STORE_TRUE.value,
            help = '是否使用 curl 文本')
        if RUN_MODE.is_command:
            item.add_argument('--url', help='请求地址')

        item.config = load_config()

        return item

    def get_completions_after_argument(self, word_for_completion):
        """
        获取补全的单词列表
        :param word_for_completion: 补全需要的单词
        """
        words = []
        if not self.argument:
            return words
        arg = self.argument
        if word_for_completion == '--name':
            # 针对 --name 参数的自动补全
            requests = self.config.get_requests(arg.module)
            words = []
            for req in requests:
                words.append(dict( text = req.name, display_meta=req.title))
            return words
        elif word_for_completion == '--module':
            modules = self.config.get_modules()
            words = [ dict(text = o.name) for o in modules ]
            return words
        elif word_for_completion == '--params':
            # 参数补全
            request = self.config.get_request(arg.module, arg.name)
            params = request.params.to_dict()
            words = self._dict_to_completions(params)
            return words
        elif word_for_completion == '--json':
            # 参数补全
            request = self.config.get_request(arg.module, arg.name)
            words = self._dict_to_completions(request.json.to_dict())
            return words

        return super().get_completions_after_argument(word_for_completion)

    def _dict_to_completions(self, data):
        words = []
        for k, v in data.items():
            words.append(dict(
                text = '{}='.format(k),
                display = '{}={}'.format(k, v),
                display_meta=''))
        return words

    def _run_command(self, args):
        """运行命令行模式"""
        # curl 模式下打开一个文件并输入文本供后续使用
        builder = RequestBuilder()
        if args.curl:
            filepath = Constants.build_tmpfile('curl')
            run_shell(f'echo "# 请输入 cUrl 文本\n" > {filepath}')
            vim_cmd = f'vim {filepath}'
            os.system(vim_cmd)
            builder = RequestBuilder.load_curl(filepath)

        if args.url:
            builder.url = args.url
            builder.format()

        if args.name:
            builder = self._get_request_builder(args)

        self.logger.info('builder {}'.format(builder.to_dict()))

        req_client = RequestClient(builder)
        res = req_client.request()
        res.print()
        History.save(res)

    def _get_request_builder(self, args):
        """获取请求构造体"""
        params = utils.list_key_val_to_dict(args.params or [])
        json_data = utils.list_key_val_to_dict(args.json or [])
        self.logger.info('arg params %s', params)
        self.logger.info('arg json %s', json_data)

        request_model = self.config.get_request(args.module, args.name)
        request_model.add_params(**params)
        # TODO 对 json 进行解析

        builder = RequestBuilder.loads_request_model(request_model,
            args.with_browser_cookie)
        return builder

    def run(self, text):
        args = self.parse_args(text)
        # 判断运行模式
        if RUN_MODE.is_command:
            self._run_command(args)
            return

        if not args.name:
            raise Exception

        builder = self._get_request_builder(args)
        request_client = RequestClient(builder)

        self._print('Space: {}'.format(args.space))
        self._print('Module: {}'.format(args.module))
        self._print('Request: {}'.format(args.name))
        self._print('Url: {}'.format(builder.url))
        self._print('Params: {}'.format(builder.params))
        self._print('Json: {}'.format(builder.json))
        self._print('请求中。。。')

        res = request_client.request()

        self._print('Status: {}'.format(res.status_code))

        History.save(res)
        if args.open:
            self._print('See in browser')
            self._open(builder.version)
        else:
            self.config.function.handler_response(
                builder, res)

    def _open(self, version):
        #  """打开请求信息"""
        port = self.config.server_port
        url = f"http://localhost:{port}/api/version/{version}"
        # TODO 适配更多系统
        os.system(f'open -a "/Applications/Google Chrome.app" "{url}"')



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""

"""
import sys
import os
import re

from wpy.argument import CommandArgumentParserFactory

from wush.argument.command import CmdArgumentParser
from wush.common.constants import Constants
from wush.common.loggers import create_logger
from wush.common.run_mode import RUN_MODE
from wush.config import load_config

from wush.cli.shell import Shell
from wush.cli.shell import init_argparse

logger = create_logger('main')

class Command(object):
    logger = create_logger('Command')

    def __init__(self, *args, **kwargs):
        if not os.path.exists(Constants.TMPDIR):
            os.makedirs(Constants.TMPDIR)
        self.config = load_config()

    #  @profile
    def run(self):
        RUN_MODE.set_command()
        parser = init_argparse()
        args = parser.parse_args()
        self.config.module_name = args.module
        self.config.space_name = args.space
        cmd = args.cmd
        if cmd == 'shell':
            shell = Shell()
            shell.run()
            return

        sys_args = sys.argv[1:]
        args_text = ' '.join(sys_args)
        # 判断 cmd 是否为 url 格式
        if re.match(r'^https?:/{2}\w.+$', cmd):
            args_text = 'run ' + args_text.replace(cmd, f'--url {cmd}')
            #  print(args_text)
            cmd = 'run'

        parser = CommandArgumentParserFactory.build_parser(cmd)
        if isinstance(parser, CmdArgumentParser):
            parser.run(args_text)




#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""

"""

from wush.web.request import RequestClient
from wush.web.request import RequestBuilder



def main():
    builder = RequestBuilder(method='get', url = 'https://ipconfig.io/json')
    client = RequestClient(builder)
    res = client.request()
    print(res.json())



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""
常量模块
"""

import os
import random

class Constants(object):
    TMPDIR = '/tmp/wush'
    BASETYPE_MAP = {
        'str': str,
        'string': str,
        'int': int,
        'list': list,
        'dict': dict,
        'float': float,
    }
    CONFIG_PATH = os.path.join(os.path.expanduser('~/.wush'), 'config.yml')
    HISTORY_PATH = os.path.expanduser('~/.wush_history')
    SERVER_PORT = '6060'

    @classmethod
    def build_tmpfile(cls, prefix):
        """创建临时文件"""
        return os.path.join(cls.TMPDIR, f'{prefix}.{random.randint(1, 90000)}')

    @classmethod
    def str_to_basetype(cls, text):
        """字符串转为基础类型"""
        if text not in cls.BASETYPE_MAP:
            raise ValueError(f'{text} is not basetype')
        return cls.BASETYPE_MAP[text]




#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy(wxnacy@gmail.com)
"""
配置模型
"""
import os

from wpy.base import BaseObject

from wush.common.constants import Constants
from wush.common.loggers import get_logger
from wush.model import datatype
from wush.model import Model
from wush.web.enums import MethodEnum
from wush.web.enums import ProtocolEnum

logger = get_logger('config.models')

class EnvModel(BaseObject):

    def __init__(self, **kwargs):
        # 将数字类型转为字符串
        for k, v in kwargs.items():
            for clz in (int, float):
                if isinstance(v, clz):
                    v = str(v)
            kwargs[k] = v
        #  for k, v in dict(os.environ).items():
            #  setattr(self, k, v)
        super().__init__(**kwargs)

class FieldModel(Model):
    """配置字段
    在配置中，单一值无法描述字段时，可以使用该模型
    """

    AUTO_FORMAT = True

    _doc = datatype.Str()
    _value = datatype.Object()
    _data_type = datatype.Object()

class AutoFieldModel(Model):
    """该模型自动使用 FieldModel"""
    AUTO_FORMAT = True
    DEFAULT_DATATYPE = datatype.Object(model = FieldModel)

    def __init__(self, **kwargs):
        """对数据进行前置过滤"""
        for k, v in kwargs.items():
            v = self._format_value(v)
            kwargs[k] = v

        super().__init__(**kwargs)

    def to_dict(self):
        """重载方法
        value 值只保留 _value 部分
        """
        logger.info('AutoFieldModel.to_dict')
        data = super().to_dict()
        for k, v in data.items():
            if isinstance(v, FieldModel):
                data[k] = v._value
            if isinstance(v, dict):
                data[k] = v.get("_value")
        return data

    def _format_value(self, v):
        if isinstance(v, dict) and '_value' in v:
            # 如果对象字段已经包含当前结构，进行格式转换
            data_type = v.get("_data_type", str)
            # 针对字符串转为基础类型
            if isinstance(data_type, str):
                data_type = Constants.str_to_basetype(data_type)
            v['_value'] = data_type(v['_value'])
        else:
            # 如果结构不对，则进行结构转换
            v = { "_value": v, "_data_type": type(v) }
        return v

    #  def __setattr__(self, key, value):
        #  value = self._format_value(value)
        #  super().__setattr__(key, value)


class RequestModel(Model):
    """请求配置模型"""
    AUTO_FORMAT = True

    name = datatype.Str()
    title = datatype.Str()
    path = datatype.Str()
    method = datatype.Str(enum = MethodEnum, default=MethodEnum.GET.value,
        upper=True)
    protocol = datatype.Str()
    domain = datatype.Str()
    cookies = datatype.Dict()
    cookie_domains = datatype.List()                # 获取 cookie 的域名列表
    headers = datatype.Dict()
    json = datatype.Object(model=AutoFieldModel)
    params = datatype.Object(model=AutoFieldModel)
    data = datatype.Str()
    url = datatype.Str()

    def add_params(self, **kwargs):
        """添加 params 参数"""
        for k, v in kwargs.items():
            origin_value = None
            if hasattr(self.params, k):
                origin_value = getattr(self.params, k)
                v = origin_value._data_type(v)
            v = self.params._format_value(v)
            setattr(self.params, k, v)
        self.params.format()


class ModuleModel(Model):
    AUTO_FORMAT = True

    name = datatype.Str()
    protocol = datatype.Str(enum = ProtocolEnum,
            default=ProtocolEnum.HTTP.value)
    domain = datatype.Str()
    url_prefix = datatype.Str()
    cookie_domains = datatype.List()                # 获取 cookie 的域名列表
    cookies = datatype.Dict()
    headers = datatype.Dict()
    requests = datatype.List(model=RequestModel)
    include = datatype.Str()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.__req__ = {o.name: o for o in self.requests}

    def get_request(self, name):
        """获取 request 模型"""
        req = self.__req__.get(name)
        if req:
            # 继承 module 的字段
            inherit_keys = ('protocol', 'domain', 'cookies', 'headers',
                'cookie_domains')
            for key in inherit_keys:
                super_value = getattr(self, key)
                sub_value = getattr(req, key)
                if not sub_value:
                    setattr(req, key, super_value)
                else:
                    # 字段结构进行拼接
                    if isinstance(super_value, dict):
                        super_value = dict(super_value)
                        super_value.update(sub_value)
                    # 列表结构进行拼接
                    if isinstance(super_value, list):
                        super_value = list(super_value)
                        super_value.extend(sub_value)

                    setattr(req, key, super_value)

            # 拼装 url
            if not req.url:
                req.url = f'{req.protocol}://{req.domain}' \
                    f'{self.url_prefix}{req.path}'
        return req

class ConfigModel(Model):
    """客户端全局配置模型"""
    modules = datatype.List(model = ModuleModel)
    env = datatype.Object(model = EnvModel)
    modules_include = datatype.List()
    server_port = datatype.Str(default = Constants.SERVER_PORT)

    def format(self):
        """重载 format
        先进行 modules 处理
        """
        super().format()
        self.__mod__ = {o.name: o for o in self.modules}

    def get_module(self, name):
        """获取模块"""
        if not self.__is_format__:
            raise Exception('ConfigModel must format first')
        module = self.__mod__.get(name)
        return module

    @classmethod
    def _iter_path(cls, filepath):
        """迭代地址"""
        if os.path.isfile(filepath):
            yield filepath
        if os.path.isdir(filepath):
            names = os.listdir(filepath)
            for _dir, _, names in os.walk(filepath):
                for name in names:
                    yield os.path.join(_dir, name)
        yield None

    def iter_module_path(self):
        """迭代模块地址列表"""
        for module_path in self.modules_include or []:
            for path in self._iter_path(module_path):
                if not path:
                    continue
                # 过滤掉不是 yml 结尾的配置文件
                if not path.endswith('.yml'):
                    continue
                yield path




#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy(wxnacy@gmail.com)
"""
配置模块
"""
import os
import yaml

from wush.config.models import ConfigModel
from wush.config.models import EnvModel
from wush.common.config_value import ConfigValue
from wush.common.constants import Constants
from wush.config.function import load_super_function
from wush.common.loggers import get_logger

class Config(object):
    logger = get_logger('Config')

    _config = None
    _function = None
    module_name = None
    space_name = None
    builtin_env = EnvModel()

    @classmethod
    def read_yml(cls, filepath):
        """读取 yml 文件"""
        with open(filepath, 'r') as f:
            return yaml.safe_load(f)

    @classmethod
    def load(cls, conf_file):
        """通过文件加载"""
        # 去读 yml 文件
        data = cls.read_yml(conf_file) or {}

        # 格式化 modules_include 地址
        config_dir = os.path.dirname(conf_file)
        full_modules_include = []
        for path in data.get("modules_include", []):
            path = os.path.join(config_dir, path)
            full_modules_include.append(path)
        data['modules_include'] = full_modules_include

        ins = cls.loads(data)
        # 获取配置目录
        #  ins._config_dir = os.path.dirname(conf_file)
        return ins

    @classmethod
    def loads(cls, data):
        """通过 dict 数据加载"""
        ins = cls()
        ins._config = ConfigModel(**data)

        # 对模块地址进行解析
        for module_path in ins._config.iter_module_path():
            module = cls.read_yml(module_path)
            ins._config.modules.append(module)

        # 格式化模型
        ins._config.format()

        # 加载方法
        ins._function = load_super_function()

        for key in ('server_port',):
            setattr(ins, key, getattr(ins._config, key))

        return ins

    def add_env(self, key, value):
        """添加环境变量"""
        setattr(self._config.env, key, value)

    @property
    def env(self):
        return self._config.env.to_dict()

    @property
    def function(self):
        return self._function

    def get_modules(self):
        """获取模块列表"""
        return self._config.modules

    def get_requests(self, module_name):
        """获取请求列表"""
        return self._config.get_module(module_name).requests

    def get_request(self, module_name, request_name):
        """获取请求模型"""
        req = self._config.get_module(module_name).get_request(request_name)
        env = self._config.env.to_dict()
        self.logger.info('config env {}'.format(env))

        # 将请求模型做环境变量格式化处理并返回
        req = ConfigValue(req).set_env(**env).format()
        req.format()
        return req

_config = None

def load_config():
    global _config
    if not _config:
        _config = Config.load(Constants.CONFIG_PATH)
    return _config

if __name__ == "__main__":
    config = load_config()



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy(wxnacy@gmail.com)
"""

"""

from wpy.base import BaseEnum

class ModeEnum(BaseEnum):
    COMMAND = 'command'
    SHELL = 'shell'

class RunMode(object):
    name = 'run_mode'
    value = None

    @property
    def is_command(self):
        """是否为命令模式"""
        return self.value == ModeEnum.COMMAND.value

    @property
    def is_shell(self):
        """是否为 shell 模式"""
        return self.value == ModeEnum.SHELL.value

    def set_command(self):
        self.value = ModeEnum.COMMAND.value

    def set_shell(self):
        self.value = ModeEnum.SHELL.value

RUN_MODE = RunMode()



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""

"""

#  import yaml
#  import os

#  from enum import Enum
#  from wush.common import constants
#  from wush.common import utils
#  from wush.common.files import FileUtils
#  from wush.common.functions import load_module
#  from wush.common.functions import load_super_function
#  from wush.common.functions import Function
#  from wush.common.functions import FunctionFactory
#  from wush.common.loggers import create_logger
#  from wush.common.exceptions import RequestException
#  from wush.models import ModuleModel
#  from wush.models import RequestModel

#  class Env():
    #  # 参数地址
    #  body_path = ''

    #  def __init__(self, **kw):
        #  self.add(**kw)

    #  def add(self, **kw):
        #  """添加环境变量"""
        #  for k, v in kw.items():
            #  setattr(self, k, v)
            #  if k.isupper():
                #  os.environ[k] = v

    #  def dict(self):
        #  data = self.__dict__
        #  return data

#  class Config():
    #  logger = create_logger('Config')

    #  _root = None
    #  env_root = ''
    #  body_root = ''
    #  module_root = ''
    #  data_root = ''
    #  #  response_root = ''
    #  #  request_root = ''
    #  #  version_root = ''
    #  space_name = ''
    #  function_modules = []
    #  modules = []
    #  _config = {}

    #  function  = None
    #  _env = Env()

    #  def __init__(self, ):
        #  """初始化"""
        #  self._root = self.get_default_root()
        #  default_config = self.get_default_config()
        #  for k, v in default_config.items():
            #  self._setattr(k, v)

    #  def _setattr(self, k, v):
        #  """设置属性"""
        #  if k == 'env':
            #  k = '_env'
            #  v = Env(**v)
        #  setattr(self, k, v)

    #  @classmethod
    #  def load(cls, fileroot):
        #  """
        #  加载配置
        #  :param fileroot: 如果是地址则获取内容，如果是数据则直接加载
        #  """
        #  cls.logger.info('Config load path: %s', fileroot)
        #  item = cls()
        #  data = {}
        #  if isinstance(fileroot, dict):
            #  data = dict(fileroot)
        #  else:
            #  item._root = fileroot
            #  filepath = os.path.join(fileroot, 'config.yml')
            #  with open(filepath, 'r') as f:
                #  data = yaml.safe_load(f)
        #  item._config = data

        #  if data:
            #  for k, v in data.items():
                #  item._setattr(k, v)

        #  # 格式化各个 root 配置
        #  for _root in ('env_root', 'module_root', 'body_root', 'data_root'):
            #  item._setattr(_root, item.fmt_path(getattr(item, _root)))

        #  item._load_functions()

        #  # 创建保持地址
        #  #  for _root in ('response_root', 'request_root', 'version_root'):
            #  #  setattr(item, _root, os.path.join(item.data_root,
                #  #  _root.replace('_root', '')))
            #  #  if not os.path.exists(getattr(item, _root)):
                #  #  os.makedirs(getattr(item, _root))

        #  return item

    #  @property
    #  def env(self):
        #  """获取 env 信息"""
        #  return self._env

    #  def _load_functions(self):
        #  """加载方法"""
        #  self.logger.info('Config functions %s', self.function_modules)
        #  if not self.function_modules:
            #  self.function = load_super_function()
            #  return
        #  for module_name in self.function_modules:
            #  load_module(module_name)

        #  functions = FunctionFactory.get_factory()
        #  self.logger.info('load functions %s', functions)

        #  f = Function(functions)
        #  self.function = f

    #  def get_module(self, module_name):
        #  """获取模块"""
        #  _config = {}
        #  if self.modules:
            #  for m in self.modules:
                #  if m.get("module") == module_name:
                    #  _config = m
        #  elif self.module_root:
            #  self.logger.info('Module: %s', module_name)
            #  _path = self.get_module_path(module_name)
            #  self.logger.info('Module path: %s', _path)
            #  if not os.path.exists(_path):
                #  raise RequestException('can not found request config {}'.format(
                    #  _path))

            #  _config = FileUtils.read_dict(_path)
        #  # 获取 env 信息
        #  _config['functions'] = self.function.get_functions()

        #  # 父 env
        #  parent_env = { "env": self.env.dict() }
        #  _config = ModuleModel._merge_config(parent_env, _config)

        #  # 获取父配置
        #  parent = _config.get("parent")
        #  if parent:
            #  parent_path = self.get_module_path(parent)
            #  parent_config = FileUtils.read_dict(parent_path) or {}
            #  ModuleModel._merge_config(parent_config, _config)
            #  _config = parent_config
        #  module = ModuleModel.load(_config)
        #  return module

    #  def set_space_name(self, space_name):
        #  self.space_name = space_name
        #  self._reload_env()

    #  def _reload_env(self):
        #  """重新加载 env"""
        #  env_path = self.get_env_path(self.space_name)
        #  self.logger.info('env_path %s', env_path)
        #  if os.path.exists(env_path):
            #  space_env = FileUtils.read_dict(env_path) or {}
            #  self.env.add(**space_env)

    #  def get_env_path(self, space_name=None):
        #  """获取 env 配置地址"""
        #  if not space_name:
            #  space_name = self.space_name
        #  if not space_name:
            #  space_name = super_function.get_current_space_name()
        #  path = os.path.join(self.env_root, '{}.yml'.format(space_name))
        #  self.logger.info('Env path %s', path)
        #  return path

    #  def get_module_path(self, module_name=None):
        #  """获取 request 配置地址"""
        #  if not module_name:
            #  module_name = self.get_default_module_name()
        #  return os.path.join(self.module_root, '{}.yml'.format(module_name))

    #  def get_body_path(self, body_name):
        #  """获取 body 配置地址"""
        #  return os.path.join(self.body_root, body_name)

    #  def get_modules(self):
        #  """获取模块名称列表"""
        #  return list(filter(lambda x: not x.startswith('.'), [
            #  o.replace('.yml', '') for o in os.listdir(self.module_root)]))

    #  def get_requests(self, module_name):
        #  """获取请求名称列表"""
        #  path = self.get_module_path(module_name)
        #  data = {}
        #  with open(path, 'r') as f:
            #  data = yaml.safe_load(f)

        #  return data.get("requests") or []

    #  def get_function(self):
        #  return self.function

    #  def dict(self):
        #  return self._config

    #  @classmethod
    #  def get_default_root(cls):
        #  return constants.CONFIG_ROOT

    #  def fmt_path(self, path):
        #  if not path:
            #  return path
        #  path = os.path.expanduser(path)
        #  if os.path.isabs(path):
            #  return path
        #  return os.path.join(self._root, path)

    #  @classmethod
    #  def get_current_body_name(cls, space_name, module_name, request_name):
        #  """获取当前 body 文件名称"""
        #  body_name = '{space}_{module}_{request}'.format(
            #  space = space_name, module=module_name, request = request_name)
        #  return body_name

    #  @classmethod
    #  def get_current_env_name(self, space_name):
        #  """获取当前 env 文件名称"""
        #  return space_name

    #  @classmethod
    #  def get_default_module_name(cls):
        #  """获取默认 module 名称"""
        #  return constants.DEFAULT_MODULE_NAME

    #  @classmethod
    #  def get_default_config(cls):
        #  """获取默认配置"""
        #  return constants.DEFAULT_CONFIG



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""

"""

import requests
import json
from datetime import datetime

from wush.common.loggers import get_logger
from wush.config.models import RequestModel
from wush.web.curl_utils import cUrl
from wush.web.cookie import Cookie
from wush.web.enums import MethodEnum
from wush.web.enums import RequestsParamsEnum
from wush.web.response import ResponseClient
from wush.model import Model
from wush.model import datatype


class RequestBuilder(Model):
    """请求构造器"""
    logger = get_logger('RequestBuilder')
    version = datatype.Str()
    method = datatype.Str(enum=MethodEnum, default=MethodEnum.GET.value,
        upper=True)                     # 请求方式
    url = datatype.Str()                # 地址
    params = datatype.Dict()            # get 请求参数
    json = datatype.Dict()              # post 请求参数
    body = datatype.Str()               # body 请求参数
    headers = datatype.Dict()           # headers 请求参数
    cookies = datatype.Dict()           # cookies 请求参数

    @classmethod
    def load_curl(cls, curl_file):
        """加载 curl 的本文文件"""
        params = cUrl.dump(curl_file)
        ins = cls(**params)
        ins.format()
        return

    @classmethod
    def loads_request_model(cls, request_model: RequestModel,
            with_browser_cookie=False):
        """加载 RequestModel 模型"""
        ins = cls()
        #  request_model.format()
        request_dict = request_model.to_dict()
        for key in RequestsParamsEnum.values():
            #  val = getattr(request_model, key)
            #  if key in ('params', 'json'):
                #  val = val.to_dict()
            val = request_dict.get(key)
            setattr(ins, key, val)

        # 对 cookie_domains 进行解析
        cookie_domains = request_model.cookie_domains
        if cookie_domains and with_browser_cookie:
            cookies  = Cookie.get_browser_cookie(*cookie_domains)
            ins.cookies = cookies

        ins.format()
        return ins

    def add_headers(self, **kwargs):
        """添加 headers"""
        self.headers.update(kwargs)

    def add_cookies(self, **kwargs):
        """添加 cookies"""
        self.cookies.update(kwargs)

    def to_requests(self):
        """转换为 requests 参数"""
        data = {}
        for key in RequestsParamsEnum.values():
            try:
                data[key] = getattr(self, key)
            except:
                pass
        return data

    def format(self):
        self.version = datetime.now().strftime('%Y%m%d%H%M%S_%s')
        super().format()


class RequestClient(object):
    """请求客户端"""
    logger = get_logger('RequestClient')

    def __init__(self, builder: RequestBuilder):
        self.builder = builder

    def request(self):
        """发送请求"""
        params = self.builder.to_requests()
        self.logger.info('request builder %s', json.dumps(params, indent=4))
        res = requests.request(**params)
        return ResponseClient(self.builder, res)




#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""
Wush
"""

import os
import requests
import json
import cgi
import traceback

from datetime import datetime
from wush.common import utils
from wush.common.loggers import create_logger
from wush.common.config import Config
from wush.web.request import RequestBuilder
from wush.web.request import RequestClient

from .models import Version
from .models import Request
from .models import Response

class Wush():
    logger = create_logger('Wush')
    module_name = ''
    request_name = ''
    space_name = ''
    config_root = ''
    _config = None
    _request = None
    _url = None
    _response = None
    is_dynamic_space = True

    def __init__(self, name=None, is_dynamic_space = True, **kw):
        if not name:
            name = __name__
        self.name = name
        self.is_dynamic_space = is_dynamic_space
        self.version = self.create_id()
        self._id = self.version
        self.config_root = Config.get_default_root()
        self.init_config(**kw)

    def reload_by_version(self, version):
        """通过版本号重新加载"""
        self.version = version
        #  data = FileUtils.read_dict(self.version_path)
        data = Version.find_one_by_id(version).to_dict()
        self.init_config(**data)

    @classmethod
    def create_id(cls):
        return datetime.now().strftime('%Y%m%d%H%M%S_%s')

    @property
    def config(self):
        """获取配置"""
        return self._config

    def _change_config(self, config_root=None, config=None):
        if self._config and not config_root and not config:
            return

        if config_root:
            self.config_root = config_root

        # 优先使用传入的 config
        if isinstance(config, dict):
            self._config = Config.load(config)

        # 在使用地址获取配置
        if not self._config and self.config_root:
            self._config = Config.load(self.config_root)

    def init_config(self,**kw):
        """初始化配置"""
        config = kw.pop('config', None)
        # config 地址
        config_root = kw.pop('config_root', None)

        # 改变配置
        self._change_config(config_root, config)

        self.logger.info('init_config kwargs %s', kw)
        for k, v in kw.items():
            if v:
                setattr(self, k, v)

        # 配置 space_name
        if not self.space_name or self.is_dynamic_space:
            self.logger.info('wush %s', self.name)
            current_space_name = self.config.get_function().get_current_space_name()
            self.logger.info('current_space_name %s', current_space_name)
            self.space_name = current_space_name

        self._config.set_space_name(self.space_name)
        self.logger.info("Space: %s", self.space_name)

        # 设置默认 module
        if not self.module_name:
            self.module_name = self.config.get_default_module_name()

        self._init_environ()
        self._build()

    def _init_environ(self):
        '''初始化环境变量'''
        current_body_name = self.config.get_current_body_name(
                self.space_name, self.module_name,self.request_name)
        current_body_path = self.config.get_body_path(current_body_name)
        self.config.env.add(**dict(
            body_path = current_body_path
        ))

    @property
    def request_builder(self):
        """获取 请求构造体"""
        return RequestBuilder(**self._request_data)

    def build(self, **kwargs):
        """
        构建请求
        :param kwargs:
        """
        for k in ('space_name', 'module_name', 'request_name'):
            val = kwargs.pop(k, None)
            if val:
                setattr(self, k, val)

        self._build(**kwargs)
        return self

    def _build(self, **kwargs):
        """获取 request
        :param dict kwargs:
            `env` 环境变量
        """
        if not self.request_name or not self.module_name:
            return
        module = self.config.get_module(self.module_name)
        request = module.get_request(self.request_name, **kwargs)
        if not request:
            return
        self._request = request
        self.logger.info('RequestModel: %s', request)
        self._url = request.url
        self._request_data = { }
        # 注入 request 必要的参数
        for k in ('method', 'url'):
            self._request_data[k] = getattr(self._request, k)
        # 注入 request 必要的参数
        for name in ('json', 'data', 'headers', 'cookies', 'params'):
            value =  getattr(self._request, name)
            if value:
                self._request_data[name] = value
        self.logger.info('Request data: %s', self._request_data)

    def request(self, **kwargs):
        """发出请求"""
        self.version = self.create_id()
        if kwargs:
            self.build(**kwargs)

        res = requests.request(**self._request_data)

        self.response_content = res.content
        self._response = res
        self.logger.info('Response')
        response_data = {}
        response_data['headers'] = dict(res.headers)
        self.logger.info(json.dumps(response_data, indent=4))
        return res

    @property
    def url(self):
        return self._url

    @property
    def response(self):
        return self._response

    def print_response(self):
        """打印返回结果"""
        print('Space:', self.space_name)
        print('Module:', self.module_name)
        print('Name:', self.request_name)
        print("Status:", self._response.status_code)
        try:
            data = self._response.json()
            try:
                data = utils.filter_json(data, self._request.filters)
            except Exception as e:
                pass
            print("Format:")
            print(json.dumps(data, indent=4, ensure_ascii=False))
        except:
            print('Content:')
            print(self._response.content)
            print('解析 json 失败')

    def get_pertty_response_content(self):
        """获取好看的 response content 内容"""
        try:
            data = self._response.json()
            try:
                data = utils.filter_json(data, self._request.filters)
            except:
                self.logger.error(traceback.format_exc())
            return json.dumps(data, indent=4, ensure_ascii=False)
        except:
            self.logger.error(traceback.format_exc())
            return self._response.content

    @property
    def response_path(self):
        """结果存储地址"""
        filename = '{}-{}'.format(self.module_name, self.request_name)
        if self._response:
            headers = dict(self._response.headers)
            content_type = headers.get("content-type") or ''
            # 处理office文件的报错
            if 'office' in content_type:
                content_disposition = headers.get("content-disposition")
                _, params = cgi.parse_header(content_disposition)
                filename = params.get("filename")

        path = os.path.join(self.config.response_root, '{}-{}'.format(
            self.version, filename))

        self.logger.info('response_path %s', path)
        return path

    def save(self):
        """保存请求配置"""
        self._save_version()
        self._save_request()
        res = Response(_id = self.version)
        try:
            res.json = json.loads(self.response_content)
        except Exception as e:
            print(e)
            res.content = self.response_content
        res.headers = dict(self.response.headers)
        res.save()

    def _save_version(self):
        data = {
            "version": self.version,
            "space_name": self.space_name,
            "module_name": self.module_name,
            "request_name": self.request_name,
            "config": self.config.dict(),
        }
        v = Version(_id = data['version'], **data)
        v.save()

    def _save_request(self):
        r = Request(_id = self.version, **self._request_data)
        r.save()

    def read(self):
        data = {
            "version": Version.find_one_by_id(self.version).to_dict(),
            "request": Request.find_one_by_id(self.version).to_dict(),
            "response": Response.find_one_by_id(self.version).to_dict(),
        }
        return data



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy(wxnacy@gmail.com)
"""
基础模块
"""
import json
from collections import defaultdict

from wpy.base import BaseObject
from wush.model.datatype import DataType
from wush.model.datatype import Object

__FILTER_SET_FIELDS__ = set(('AUTO_FORMAT', 'DEFAULT_DATATYPE',
    '__is_format__', '__datatype_fields__'))

class Model(BaseObject):

    AUTO_FORMAT = False     # 是否自动 format
    # 字段默认的 datatype
    DEFAULT_DATATYPE = None

    # 装载 Model 的 datatype 类型字段
    __datatype_fields__ = defaultdict(dict)
    # 对象是否已经进行过 format 操作
    __is_format__ = False

    def __init__(self, **kwargs):
        #  self.__is_format__ = False
        self.__init_datatype_fields()
        super().__init__(**kwargs)

        # 将未赋值的字段设置为 None
        for key in self.__get_datatype_fields().keys():
            if hasattr(self, key) and isinstance(getattr(self, key), DataType):
                setattr(self, key, None)

        # 判断自动 format
        if self.AUTO_FORMAT:
            self.format()

    def __is_need_format_setattr__(self, key, value):
        """是否需要格式化 __setattr__"""
        if not self.DEFAULT_DATATYPE:
            return False
        if key in __FILTER_SET_FIELDS__:
            return False

        return True

    def __setattr__(self, key, value):
        #  """重载设置值的方法"""
        # 如果字段的默认模型存在，则设置值之前先添加 datatype
        if self.DEFAULT_DATATYPE:
            default_datatype = self.__build_default_datatype()
            self.__add_datatype_fields(**{ key: default_datatype })

        super().__setattr__(key, value)

    @classmethod
    def __add_datatype_fields(cls, **kwargs):
        """添加 datatype 字段"""
        for k, v in kwargs.items():
            if k in __FILTER_SET_FIELDS__:
                continue
            if not isinstance(v, DataType):
                continue
            # 设置 datatype 名称
            v._name = k
            if k not in cls.__datatype_fields__[cls]:
                cls.__datatype_fields__[cls][k] = v

    @classmethod
    def __get_datatype_fields(cls):
        """获取 datatype 字段"""
        return cls.__datatype_fields__.get(cls, {})

    @classmethod
    def __init_datatype_fields(cls):
        '''获取默认 dict'''
        classes = [cls]
        # 兼容父类的 __dict__
        #  classes.extend(cls.__bases__)
        for clz in classes:
            cls.__add_datatype_fields(**clz.__dict__)

    def __build_default_datatype(self):
        """构建默认的 datatype"""
        return self.DEFAULT_DATATYPE

    def format(self):
        """将 datatype 字段进行格式化处理"""
        self._format(self)

    def _format(self, model):
        """嵌套 format"""
        # 判断是否有默认字段
        # 并对对象中存在的赋值添加到 __datatype_fields__ 中
        if model.DEFAULT_DATATYPE:
            for key in model.__dict__.keys():
                # 符合条件的 key 才会进行添加 datatype 处理
                default_datatype = model.__build_default_datatype()
                model.__add_datatype_fields(**{key: default_datatype })

        for k, v in model.__get_datatype_fields().items():
            # 格式化单个字段
            self._format_field(model, k)

        model.__is_format__ = True

    def _format_field(self, model, field):
        """格式化 field"""
        # 如果 field 不是 datatype 类型，直接返回
        datatype_fields = model.__get_datatype_fields()
        dt_ins = datatype_fields.get(field)
        if not dt_ins:
            return

        try:
            dt_ins.set_value(getattr(model, field))
        except:
            pass
        # 校验
        dt_ins.valid()
        set_val = dt_ins.value()
        setattr(model, field, set_val)
        # 嵌套 format
        if isinstance(set_val, Model):
            #  self._format(set_val)
            set_val.format()

    def to_dict(self):
        """
        将实例转为 dict 数据
        执行前需要保证实例已经执行过 format 方法或设置成员变量 AUTO_FORMAT=True
        """
        return self._to_dict(self)

    @classmethod
    def _to_dict(cls, model):
        """对 Model 进行循环 to_dict 操作"""
        data = {}
        for key in model.__get_datatype_fields().keys():
            if not hasattr(model, key):
                continue
            origin_value = getattr(model, key)
            if isinstance(origin_value, Model):
                data[key] = origin_value.to_dict()
            else:
                data[key] = origin_value
        return data

    def to_json(self):
        return json.dumps(self, default=lambda o: o.to_dict(), sort_keys=True)

from wush.model import datatype
class Book(Model):
    name = datatype.Str(default='book')

class User(Model):
    AUTO_FORMAT = True
    DEFAULT_DATATYPE = Object(model = Book)
    #  DEFAULT_DATATYPE = Book
    #  book = datatype.Str()

class Field(Model):
    doc = datatype.Str()
    _type = datatype.Object()
    value = datatype.Object()

class Json(Model):
    DEFAULT_DATATYPE = datatype.Object(model = Field)

    cust_field = datatype.Str(default='cust')

class Request(Model):
    json = datatype.Object(model=Json)

if __name__ == "__main__":

    #  u = User()
    #  u.name = { "name": 'wxnacy' }
    #  u.format()
    #  assert u.name.name == 'wxnacy'

    #  b = Book()
    #  b.format()
    #  b.name = 'books'
    #  u = User()
    #  u.name = 'wxnacy'
    
    data = { "json": { "id": { "_value": 1 }, "name": { "_value": "wxnacy" } } }
    r = Request(**data)
    r.format()
    assert r.json.id._value == 1
    assert r.json.name._value == 'wxnacy'
    # 手动设置的字段，使用原类型
    assert r.json.cust_field == 'cust'



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
请求历史
"""


from lfsdb import FSModel
from lfsdb import FSColumn

from wush.web.response import ResponseClient

class HistoryModel(FSModel):
    db = 'wush'
    table = 'history'

    version = FSColumn(str)
    request = FSColumn(dict)
    response = FSColumn(dict)

class History(object):

    @classmethod
    def save(cls, response_client: ResponseClient):

        h = HistoryModel()
        h.version = response_client.request_builder.version
        h._id = h.version
        h.request = response_client.request_builder.to_dict()
        res = {
            'is_json': response_client.is_json(),
            'json': response_client.json(),
            'text': response_client.text
        }
        h.response = res
        h.save()

    @classmethod
    def read(cls, version):
        h = HistoryModel.find_one_by_id(version)
        return h.to_dict()
